@@ -21,6 +21,7 @@ bool CID3v2Tag::Analyze(unsigned char * pTagData, int nTagBytes)
     m_nTagBytes = nTagBytes;
 
     // over allocate a little so we don't have to worry about buffer size as we read the data
+    // we also zero the buffer so doing string length tests in the buffer will always be safe (string could be long, but won't ever be infinite)
     m_spTag.AllocateArray(nTagBytes + 1024, true);
 
     // copy tag data
@@ -116,7 +117,7 @@ bool CID3v2Tag::Analyze(unsigned char * pTagData, int nTagBytes)
                     ((nFrameBytes & 0x7F000000) >> 3);
             }
 
-            CSmartPtr<str_utfn> * pTag = NULL;
+            CSmartPtr<str_utfn> * pTag = APE_NULL;
             if ((m_spTag[z + 0] == 'T') && (m_spTag[z + 1] == 'I') && (m_spTag[z + 2] == 'T') && (m_spTag[z + 3] == '2'))
                 pTag = &m_spTitle;
             else if ((m_spTag[z + 0] == 'T') && (m_spTag[z + 1] == 'P') && (m_spTag[z + 2] == 'E') && (m_spTag[z + 3] == '1'))
@@ -135,12 +136,7 @@ bool CID3v2Tag::Analyze(unsigned char * pTagData, int nTagBytes)
             if ((pTag != APE_NULL) && (pTag->GetPtr() == APE_NULL))
             {
                 // found a tag
-                int nEncoding = m_spTag[z + 10];
-                unsigned char * pStart = &m_spTag[z + 11];
-
-                // decode
-                ID3v2TextEncoding Encoding = static_cast<ID3v2TextEncoding>(nEncoding);
-                ConvertID3v2String(pTag, Encoding, pStart);
+                ReadTag(z, nFrameBytes, pTag);
             }
             else if ((m_spTag[z + 0] == 'A') && (m_spTag[z + 1] == 'P') && (m_spTag[z + 2] == 'I') && (m_spTag[z + 3] == 'C') && (m_spImage == APE_NULL))
             {
@@ -163,11 +159,15 @@ bool CID3v2Tag::Analyze(unsigned char * pTagData, int nTagBytes)
             }
             else if ((m_spTag[z + 0] == 'T') && (m_spTag[z + 1] == 'R') && (m_spTag[z + 2] == 'C') && (m_spTag[z + 3] == 'K'))
             {
-                m_nTrackNumber = atoi(reinterpret_cast<char *>(&m_spTag[z + 11]));
+                CSmartPtr<str_utfn> spNumber;
+                ReadTag(z, nFrameBytes, &spNumber);
+                m_nTrackNumber = _wtoi(spNumber);
             }
             else if ((m_spTag[z + 0] == 'T') && (m_spTag[z + 1] == 'P') && (m_spTag[z + 2] == 'O') && (m_spTag[z + 3] == 'S'))
             {
-                m_nDiscNumber = atoi(reinterpret_cast<char*>(&m_spTag[z + 11]));
+                CSmartPtr<str_utfn> spNumber;
+                ReadTag(z, nFrameBytes, &spNumber);
+                m_nDiscNumber = _wtoi(spNumber);
             }
             else if ((m_spTag[z + 0] == 'C') && (m_spTag[z + 1] == 'O') && (m_spTag[z + 2] == 'M') && (m_spTag[z + 3] == 'M'))
             {
@@ -197,7 +197,7 @@ bool CID3v2Tag::Analyze(unsigned char * pTagData, int nTagBytes)
                     CSmartPtr<str_utfn> spCommentValue;
                     ConvertID3v2String(&spCommentValue, Encoding, pStart + nDescriptionBytes);
 
-                    if (m_spComment == NULL)
+                    if (m_spComment == APE_NULL)
                     {
                         // no comment yet, so use this string
                         m_spComment.Assign(spCommentValue.GetPtr(), true);
@@ -208,7 +208,7 @@ bool CID3v2Tag::Analyze(unsigned char * pTagData, int nTagBytes)
                         // append this string with a semi-colon between the values
                         CSmartPtr<str_utfn> spCommentFull;
                         size_t nCharacters = wcslen(m_spComment) + 1 + wcslen(spCommentValue) + 1;
-                        spCommentFull.AllocateArray(nCharacters);
+                        spCommentFull.AllocateArray(static_cast<int64>(nCharacters));
                         wcscpy_s(spCommentFull, nCharacters, m_spComment);
                         wcscat_s(spCommentFull, nCharacters, L";");
                         wcscat_s(spCommentFull, nCharacters, spCommentValue);
@@ -229,22 +229,36 @@ bool CID3v2Tag::Analyze(unsigned char * pTagData, int nTagBytes)
     return m_bFoundTag;
 }
 
-void CID3v2Tag::ConvertID3v2String(CSmartPtr<str_utfn> * pTag, ID3v2TextEncoding Encoding, unsigned char * pStart, int * pnLengthBytes)
+void CID3v2Tag::ReadTag(int z, uint32 nFrameBytes, CSmartPtr<str_utfn> * pTag)
+{
+    // found a tag
+    int nEncoding = m_spTag[z + 10];
+    unsigned char * pStart = &m_spTag[z + 11];
+
+    // get size of value
+    int nValueBytes = static_cast<int>(nFrameBytes) - 1;
+
+    // decode
+    ID3v2TextEncoding Encoding = static_cast<ID3v2TextEncoding>(nEncoding);
+    ConvertID3v2String(pTag, Encoding, pStart, APE_NULL, nValueBytes);
+}
+
+void CID3v2Tag::ConvertID3v2String(CSmartPtr<str_utfn> * pTag, ID3v2TextEncoding Encoding, unsigned char * pStart, int * pnLengthBytes, int nBytes)
 {
     if (Encoding == ISO_8859_1)
     {
-        int nLength = static_cast<int>(strlen(reinterpret_cast<char *>(pStart)));
+        int nLength = (nBytes >= 0) ? nBytes : static_cast<int>(strlen(reinterpret_cast<char*>(pStart)));
         CSmartPtr<str_ansi> spTagValueANSI(new str_ansi[static_cast<size_t>(nLength + 1)], true);
-        memset(spTagValueANSI, 0, static_cast<size_t>(nLength + 1));
+        APE_CLEAR_ARRAY(spTagValueANSI, nLength + 1);
         memcpy(spTagValueANSI.GetPtr(), reinterpret_cast<char *>(pStart), static_cast<size_t>(nLength));
         pTag->Assign(CAPECharacterHelper::GetUTFNFromANSI(spTagValueANSI.GetPtr()), true);
         if (pnLengthBytes) *pnLengthBytes = nLength + 1;
     }
     else if (Encoding == UTF_8)
     {
-        int nLength = static_cast<int>(strlen(reinterpret_cast<char *>(pStart)));
+        int nLength = (nBytes >= 0) ? nBytes : static_cast<int>(strlen(reinterpret_cast<char*>(pStart)));
         CSmartPtr<str_utf8> spTagValueUTF8(new str_utf8[static_cast<size_t>(nLength + 1)], true);
-        memset(spTagValueUTF8, 0, static_cast<size_t>(nLength + 1));
+        APE_CLEAR_ARRAY(spTagValueUTF8, nLength + 1);
         memcpy(spTagValueUTF8.GetPtr(), reinterpret_cast<char*>(pStart), static_cast<size_t>(nLength));
         pTag->Assign(CAPECharacterHelper::GetUTFNFromUTF8(spTagValueUTF8.GetPtr()), true);
         if (pnLengthBytes) *pnLengthBytes = nLength + 1;
@@ -257,10 +271,11 @@ void CID3v2Tag::ConvertID3v2String(CSmartPtr<str_utfn> * pTag, ID3v2TextEncoding
         {
             pStart += 2;
             pString = reinterpret_cast<str_utfn *>(pStart);
+            nBytes -= 2;
         }
-        int nLength = static_cast<int>(wcslen(pString));
+        int nLength = (nBytes >= 0) ? (nBytes / static_cast<int>(sizeof(str_utfn))) : static_cast<int>(wcslen(pString));
         CSmartPtr<str_utfn> spTagValueUTFN(new str_utfn[static_cast<size_t>(nLength + 1)], true);
-        memset(spTagValueUTFN, 0, static_cast<size_t>(nLength + 1) * sizeof(str_utfn));
+        APE_CLEAR_ARRAY(spTagValueUTFN, nLength + 1);
         memcpy(spTagValueUTFN.GetPtr(), pStart, static_cast<size_t>(nLength) * sizeof(str_utfn));
         pTag->Assign(spTagValueUTFN, true);
         spTagValueUTFN.SetDelete(false);
@@ -274,10 +289,11 @@ void CID3v2Tag::ConvertID3v2String(CSmartPtr<str_utfn> * pTag, ID3v2TextEncoding
         {
             pStart += 2;
             pString = reinterpret_cast<str_utfn*>(pStart);
+            nBytes -= 2;
         }
-        int nLength = static_cast<int>(wcslen(pString));
+        int nLength = (nBytes >= 0) ? (nBytes / static_cast<int>(sizeof(str_utfn))) : static_cast<int>(wcslen(pString));
         CSmartPtr<str_utfn> spTagValueUTFN(new str_utfn[static_cast<size_t>(nLength + 1)], true);
-        memset(spTagValueUTFN, 0, static_cast<size_t>(nLength + 1) * sizeof(str_utfn));
+        APE_CLEAR_ARRAY(spTagValueUTFN, nLength + 1);
         memcpy(spTagValueUTFN.GetPtr(), pStart, static_cast<size_t>(nLength) * sizeof(str_utfn));
         SwitchBufferBytes(spTagValueUTFN.GetPtr(), 2, nLength);
         pTag->Assign(spTagValueUTFN, true);
